"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execQuery = exports.composeMetricSumQuery = exports.composeMetricDistributionQuery = exports.composeMetricsTableBaseFilter = exports.composeBaseQueryBody = exports.parseQuery = exports.addTimestampRangeToBin = void 0;
const util_1 = require("./util");
const coronerdUtils_1 = require("./coronerdUtils");
const index_1 = require("../model/query/index");
const filter_1 = require("../model/query/filter");
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const queryItem_1 = require("../model/queryItem");
function addTimestampRangeToBin(query, filters) {
    var _a, _b, _c, _d, _e;
    const timestampBin = (_b = (_a = query === null || query === void 0 ? void 0 : query.fold) === null || _a === void 0 ? void 0 : _a.timestamp) === null || _b === void 0 ? void 0 : _b.find((t) => t[0] === attributeAggregations_1.AggregationOp.BIN);
    if (timestampBin && timestampBin.length === 2) {
        const timestampRange = (_c = filters[0]) === null || _c === void 0 ? void 0 : _c.timestamp;
        const rangeStart = (_d = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_LEAST)) === null || _d === void 0 ? void 0 : _d[1];
        const rangeEnd = (_e = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_MOST)) === null || _e === void 0 ? void 0 : _e[1];
        if (rangeStart !== undefined && rangeEnd !== undefined) {
            const newTimestampBinAgg = [
                attributeAggregations_1.AggregationOp.BIN,
                timestampBin[1],
                rangeStart,
                rangeEnd,
            ];
            index_1.removeAggregation(query, 'timestamp', attributeAggregations_1.AggregationOp.BIN);
            index_1.addAggregation(query, 'timestamp', newTimestampBinAgg);
        }
    }
}
exports.addTimestampRangeToBin = addTimestampRangeToBin;
function getOverrideAttributeToken(attribute) {
    return `<%${attribute.trim()}%>`;
}
function parseQuery(originalQueryJson, versionAttribute) {
    let queryJson;
    if (versionAttribute) {
        const overrideTokenTarget = getOverrideAttributeToken('versionAttribute');
        queryJson = util_1.replaceAll(originalQueryJson, versionAttribute, overrideTokenTarget);
    }
    else {
        queryJson = originalQueryJson;
    }
    return JSON.parse(queryJson);
}
exports.parseQuery = parseQuery;
function composeBaseQueryBody(originalQueryJson, filters, order, versionAttribute, group) {
    const query = parseQuery(originalQueryJson, versionAttribute);
    query.filter = filters;
    query.order = order;
    if (group && Array.isArray(group) && group.length === 1) {
        query.group = group;
    }
    addTimestampRangeToBin(query, filters);
    return query;
}
exports.composeBaseQueryBody = composeBaseQueryBody;
async function composeMetricsTableBaseFilter(coronerUrl, token, project, filters) {
    var _a;
    const describeResp = await coronerdUtils_1.fetchDescribe(coronerUrl, token, project, 'metrics');
    const metricAttributes = describeResp.map(function (attribute) {
        return attribute.name;
    });
    const timestampFilters = (_a = filters[0]) === null || _a === void 0 ? void 0 : _a.timestamp;
    const newFilterObject = { timestamp: timestampFilters };
    Object.keys(filters[0]).forEach(function (filterAttribute) {
        if (metricAttributes.includes(filterAttribute)) {
            newFilterObject[filterAttribute] = filters[0][filterAttribute];
        }
    });
    return newFilterObject;
}
exports.composeMetricsTableBaseFilter = composeMetricsTableBaseFilter;
async function composeMetricDistributionQuery(coronerUrl, token, project, filters) {
    const query = index_1.getEmptyQuery();
    const newFilterObject = await composeMetricsTableBaseFilter(coronerUrl, token, project, filters);
    query.filter = [newFilterObject];
    query.group = ['*'];
    query.fold = { _metric_group: [[attributeAggregations_1.AggregationOp.DISTRIBUTION, 50]] };
    query.table = 'metrics';
    return query;
}
exports.composeMetricDistributionQuery = composeMetricDistributionQuery;
async function composeMetricSumQuery(coronerUrl, token, project, filters, metricGroup) {
    const query = index_1.getEmptyQuery();
    const newFilterObject = await composeMetricsTableBaseFilter(coronerUrl, token, project, filters);
    newFilterObject._metric_group = [[filter_1.Operater.EQUAL, metricGroup]];
    query.filter = [newFilterObject];
    query.group = ['*'];
    query.fold = { _value: [[attributeAggregations_1.AggregationOp.SUM]] };
    query.table = 'metrics';
    return query;
}
exports.composeMetricSumQuery = composeMetricSumQuery;
async function metricsSumPointQueryFetch(coronerUrl, token, project, universe, filters, config) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u;
    const queryForDistributionRequest = await composeMetricDistributionQuery(coronerUrl, token, project, filters);
    const crdbDistributionResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForDistributionRequest));
    const topMetricGroup = (_j = (_h = (_g = (_f = (_e = (_d = (_c = (_b = (_a = crdbDistributionResp.response) === null || _a === void 0 ? void 0 : _a.response) === null || _b === void 0 ? void 0 : _b.values) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d[1]) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.vals) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j[0];
    if (topMetricGroup) {
        const queryForSumRequest = await composeMetricSumQuery(coronerUrl, token, project, filters, topMetricGroup);
        const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForSumRequest));
        const count = (_q = (_p = (_o = (_m = (_l = (_k = crdbSumResp.response) === null || _k === void 0 ? void 0 : _k.response) === null || _l === void 0 ? void 0 : _l.values) === null || _m === void 0 ? void 0 : _m[0]) === null || _o === void 0 ? void 0 : _o[1]) === null || _p === void 0 ? void 0 : _p[0]) === null || _q === void 0 ? void 0 : _q[0];
        return {
            response: {
                group: topMetricGroup,
                count: count ? count : 0,
            },
            status: crdbSumResp.status,
            statusText: crdbSumResp.statusText,
        };
    }
    else {
        const currentProjectObject = (_t = (_s = (_r = config === null || config === void 0 ? void 0 : config.universes) === null || _r === void 0 ? void 0 : _r[universe]) === null || _s === void 0 ? void 0 : _s.projects_ext) === null || _t === void 0 ? void 0 : _t.find(function (project) {
            return project.name === project;
        });
        const metricGroups = await coronerdUtils_1.fetchBPGGet(coronerUrl, token, coronerdUtils_1.GET_METRICS_GROUPS);
        const metricGroupForProject = metricGroups.filter(function (metricGroup) {
            return metricGroup.project === currentProjectObject.pid;
        });
        const bpgMetricGroup = (_u = metricGroupForProject === null || metricGroupForProject === void 0 ? void 0 : metricGroupForProject[0]) === null || _u === void 0 ? void 0 : _u.name;
        if (bpgMetricGroup) {
            return {
                response: { group: bpgMetricGroup, count: 0 },
                status: 200,
                statusText: 'success',
            };
        }
        else {
            return {
                response: { group: null, count: null },
                status: 200,
                statusText: 'no metric group found for project',
            };
        }
    }
}
async function basicQueryFetch(coronerUrl, token, project, queryItem, queryOverrides) {
    const { filter, order, versionAttribute, group } = queryOverrides;
    const queryForRequest = composeBaseQueryBody(queryItem.query, filter, order, versionAttribute, group);
    const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForRequest));
    return [queryForRequest, crdbResp];
}
async function execQuery(queryItem, queryOverrides, coronerUrl, token, project, universe, config) {
    switch (queryItem.variant) {
        case queryItem_1.Variant.SUMMED_EVENT_POINT: {
            const crdbResp = await metricsSumPointQueryFetch(coronerUrl, token, project, universe, queryOverrides.filter, config);
            return [undefined, crdbResp];
        }
        default:
            return await basicQueryFetch(coronerUrl, token, project, queryItem, queryOverrides);
    }
}
exports.execQuery = execQuery;
//# sourceMappingURL=queryUtils.js.map