const fs = require('fs');
const path = require('path');
const { BacktraceService, Logger } = require('backtrace-service');
const { isStandalone, getConfPath } = require('./dist/lib/util');

const log = Logger.getLogger();

const configDb = BacktraceService.readConfiguration(
  'overview',
  6457,
  isStandalone ? getConfPath('overview') : undefined,
  true,
).database;

// Check db directory if it does not already exist
const directory = path.dirname(configDb.connection.filename);
if (!fs.existsSync(directory)) {
  fs.mkdirSync(directory, { recursive: true }, (err) => {
    if (err) {
      log.error(`Error: Database directory ${directory} cannot be created.`);
      throw err;
    }
  });
}

const defaults = {
  migrations: {
    directory: `${__dirname}/migrations`,
  },
  useNullAsDefault: true,
};

module.exports = Object.assign(defaults, configDb);
